package il.ac.tau.cs.software1.ex3;


/**
 * An instance of this class represent a department in a company.
 * A department can have at most one parent department.
 * A department can have workers and sub-departments.
 *
 */
public class Department {
	private String m_name;
	private Department m_parentDepartment = null;
	
	
	private Worker[] m_workers = new Worker[0];
			
	private Department[] m_subDepartments = new Department[0];
		
	/**
	 * Constructor.
	 * @param name the name of the department. The name must be unique within the company.
	 */
	public Department(String name) {
		m_name = name;
	}

	/**
	 * 
	 * @return the name of the department
	 */
	public String getName() {
		return m_name;
	}
	
	public String toString() {
		return getName();
	}

	public boolean equals(Department otherTeam) {
		return getName().equals(otherTeam.getName());
	}
	
	/**
	 * prints the department name, its workers and subdepartment in a formatted way.
	 * @param indentation
	 */
	public void print(String indentation){
		System.out.println(indentation+getName());
		if (m_workers.length > 0){
			System.out.println(indentation+ "Workers ("+m_workers.length+"):");
			for (Worker worker : m_workers){
				System.out.println(indentation+"\t"+worker);
			}
		}
		if (m_subDepartments.length > 0){
			System.out.println(indentation+"Departments ("+m_subDepartments.length+"):");
			for (Department subDepartment: m_subDepartments){
				subDepartment.print(indentation+"\t");
			}
		}
	}
	
	/**
	 * 
	 * @return the name of the department to which this department belongs. If the department
	 * does not belong to any department - returns null;
	 */
	public Department getParentDepartment(){
		return m_parentDepartment;
	}
	/**
	 * 
	 * @return an array containing the list of workers in the department.
	 */
	public Worker[] getWorkers() {		
		return m_workers;
	}
	
	/**
	 * Adds a new worker to the department. If the given worker already exists,
	 * no action will be taken.
	 * 
	 * @param worker the new worker
	 */
	public void addWorker(Worker worker) {
		if (findWorkerIdx(worker) != -1) {
			return;
		}
		Worker[] oldWorkers = m_workers;
		m_workers = new Worker[oldWorkers.length+1];		
		System.arraycopy(oldWorkers,0,m_workers,0,oldWorkers.length);
		m_workers[m_workers.length-1] = worker;
		
		worker.setDepartment(this);
	}
	
	/**
	 * Removes a worker from the department. 
	 * @param worker
	 */
	public void removeWorker(Worker worker) {
		int i = findWorkerIdx(worker);
		if (i == -1) {
			return;
		}
		Worker[] oldWorkers = m_workers;
		m_workers = new Worker[m_workers.length-1];
		System.arraycopy(oldWorkers,0,m_workers,0,i);
		if (i < oldWorkers.length-1){
			System.arraycopy(oldWorkers,i+1,m_workers,i,oldWorkers.length-i-1);
		}
	}
	
	/**
	 * 
	 * @return an array containing the list of sub-departments.
	 */
	public Department[] getSubDepartments(){
		return m_subDepartments;
	}
	
	/**
	 * Adds a new department as a sub-department. If the department already exists
	 * as a sub-department, no action would be taken.
	 * @param department the new department
	 */
	public void addSubDepartment(Department department) {
		if (findSubDepartmentIdx(department) != -1) {
			return;
		}
		department.setParentDepartment(this);
		
		Department[] oldSubDepartments = m_subDepartments;
		m_subDepartments = new Department[oldSubDepartments.length+1];
		System.arraycopy(oldSubDepartments,0,m_subDepartments,0,oldSubDepartments.length);
		m_subDepartments[m_subDepartments.length-1] = department;		
	}
	
	/**
	 * Removes the given department from the list of sub-departments. 
	 * @param department
	 */
	public void removeSubDepartment(Department department) {
		int i = findSubDepartmentIdx(department);
		if (i == -1) {
			return;
		}
		
		Department[] oldSubDepartment = m_subDepartments;
		m_subDepartments = new Department[m_subDepartments.length-1];
		System.arraycopy(oldSubDepartment,0,m_subDepartments,0,i);
		if (i < oldSubDepartment.length-1){
			System.arraycopy(oldSubDepartment,i+1,m_subDepartments,i,oldSubDepartment.length-i-1);
		}
	}
	
	private void setParentDepartment(Department department) {
		if (m_parentDepartment != department) {
			if (m_parentDepartment != null) {
				m_parentDepartment.removeSubDepartment(this);
			}
			m_parentDepartment = department;
		}	
	}
		
	private int findWorkerIdx(Worker worker) {
		for (int i = m_workers.length - 1; i > -1; i--) {
			if (m_workers[i].equals(worker)) {
				return i;
			}
		}
		return -1;
	}
	
	private int findSubDepartmentIdx(Department department) {
		for (int i = m_subDepartments.length - 1; i > -1; i--) {
			if (m_subDepartments[i].equals(department)) {
				return i;
			}
		}
		return -1;
	}	
}
